@@ -11,6 +11,7 @@
 #define LLVM_CLANG_FRONTEND_COMPILERINSTANCE_H_
 
 #include "clang/Frontend/CompilerInvocation.h"
+#include "llvm/ADT/IntrusiveRefCntPtr.h"
 #include "llvm/ADT/StringRef.h"
 #include "llvm/ADT/OwningPtr.h"
 #include <cassert>
@@ -33,8 +34,9 @@ class DiagnosticClient;
 class ExternalASTSource;
 class FileManager;
 class FrontendAction;
+class ASTReader;
 class Preprocessor;
-class Source;
+class Sema;
 class SourceManager;
 class TargetInfo;
 
@@ -64,10 +66,7 @@ class CompilerInstance {
   llvm::OwningPtr<CompilerInvocation> Invocation;
 
   /// The diagnostics engine instance.
-  llvm::OwningPtr<Diagnostic> Diagnostics;
-
-  /// The diagnostics client instance.
-  llvm::OwningPtr<DiagnosticClient> DiagClient;
+  llvm::IntrusiveRefCntPtr<Diagnostic> Diagnostics;
 
   /// The target being compiled for.
   llvm::OwningPtr<TargetInfo> Target;
@@ -90,6 +89,9 @@ class CompilerInstance {
   /// The code completion consumer.
   llvm::OwningPtr<CodeCompleteConsumer> CompletionConsumer;
 
+  /// \brief The semantic analysis object.
+  llvm::OwningPtr<Sema> TheSema;
+  
   /// The frontend timer
   llvm::OwningPtr<llvm::Timer> FrontendTimer;
 
@@ -256,27 +258,16 @@ public:
     return *Diagnostics;
   }
 
-  /// takeDiagnostics - Remove the current diagnostics engine and give ownership
-  /// to the caller.
-  Diagnostic *takeDiagnostics() { return Diagnostics.take(); }
-
   /// setDiagnostics - Replace the current diagnostics engine; the compiler
   /// instance takes ownership of \arg Value.
   void setDiagnostics(Diagnostic *Value);
 
   DiagnosticClient &getDiagnosticClient() const {
-    assert(DiagClient && "Compiler instance has no diagnostic client!");
-    return *DiagClient;
+    assert(Diagnostics && Diagnostics->getClient() && 
+           "Compiler instance has no diagnostic client!");
+    return *Diagnostics->getClient();
   }
 
-  /// takeDiagnosticClient - Remove the current diagnostics client and give
-  /// ownership to the caller.
-  DiagnosticClient *takeDiagnosticClient() { return DiagClient.take(); }
-
-  /// setDiagnosticClient - Replace the current diagnostics client; the compiler
-  /// instance takes ownership of \arg Value.
-  void setDiagnosticClient(DiagnosticClient *Value);
-
   /// }
   /// @name Target Info
   /// {
@@ -372,6 +363,10 @@ public:
   /// takes ownership of \arg Value.
   void setASTContext(ASTContext *Value);
 
+  /// \brief Replace the current Sema; the compiler instance takes ownership
+  /// of S.
+  void setSema(Sema *S);
+  
   /// }
   /// @name ASTConsumer
   /// {
@@ -391,6 +386,18 @@ public:
   /// takes ownership of \arg Value.
   void setASTConsumer(ASTConsumer *Value);
 
+  /// }
+  /// @name Semantic analysis
+  /// {
+  bool hasSema() const { return TheSema != 0; }
+  
+  Sema &getSema() const { 
+    assert(TheSema && "Compiler instance has no Sema object!");
+    return *TheSema;
+  }
+  
+  Sema *takeSema() { return TheSema.take(); }
+  
   /// }
   /// @name Code Completion
   /// {
@@ -470,8 +477,8 @@ public:
   /// must extend past that of the diagnostic engine.
   ///
   /// \return The new object on success, or null on failure.
-  static Diagnostic *createDiagnostics(const DiagnosticOptions &Opts,
-                                       int Argc, char **Argv);
+  static llvm::IntrusiveRefCntPtr<Diagnostic> 
+  createDiagnostics(const DiagnosticOptions &Opts, int Argc, char **Argv);
 
   /// Create the file manager and replace any existing one with it.
   void createFileManager();
@@ -502,14 +509,18 @@ public:
 
   /// Create an external AST source to read a PCH file and attach it to the AST
   /// context.
-  void createPCHExternalASTSource(llvm::StringRef Path);
+  void createPCHExternalASTSource(llvm::StringRef Path,
+                                  bool DisablePCHValidation,
+                                  void *DeserializationListener);
 
   /// Create an external AST source to read a PCH file.
   ///
   /// \return - The new object on success, or null on failure.
   static ExternalASTSource *
   createPCHExternalASTSource(llvm::StringRef Path, const std::string &Sysroot,
-                             Preprocessor &PP, ASTContext &Context);
+                             bool DisablePCHValidation,
+                             Preprocessor &PP, ASTContext &Context,
+                             void *DeserializationListener);
 
   /// Create a code completion consumer using the invocation; note that this
   /// will cause the source manager to truncate the input source file at the
@@ -523,8 +534,13 @@ public:
   createCodeCompletionConsumer(Preprocessor &PP, const std::string &Filename,
                                unsigned Line, unsigned Column,
                                bool UseDebugPrinter, bool ShowMacros,
+                               bool ShowCodePatterns, bool ShowGlobals,
                                llvm::raw_ostream &OS);
 
+  /// \brief Create the Sema object to be used for parsing.
+  void createSema(bool CompleteTranslationUnit,
+                  CodeCompleteConsumer *CompletionConsumer);
+  
   /// Create the frontend timer and replace any existing one with it.
   void createFrontendTimer();
 
